# -*- coding: utf-8 -*-
"""
/***************************************************************************
 RTKubeDockWidget
                                 A QGIS plugin
 RTKube interface
                             -------------------
        begin                : 2018-07-17
        git sha              : $Format:%H$
        copyright            : (C) 2018 by Oslandia
        email                : infos@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from __future__ import absolute_import
from builtins import str
import os
import time
import logging
from logging.handlers import TimedRotatingFileHandler
from qgis.PyQt import QtWidgets, uic, QtCore

from qgis.utils import iface
from qgis.core import (Qgis,
                       QgsGeometry,
                       QgsCoordinateReferenceSystem,
                       QgsCoordinateTransform,
                       QgsPointXY,
                       QgsProject)


from .constantes import *
from .rtkubeSerial import rtkubeSerial

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'rtkube_dockwidget_base.ui'))

NB_FIELDS_DATA = 10


class RTKubeDockWidget(QtWidgets.QDockWidget, FORM_CLASS):

    closingPlugin = QtCore.pyqtSignal()

    # send data to other plugins
    dataToSMU = QtCore.pyqtSignal(str,  # date
                                  float,  # x
                                  float,  # y
                                  float,  # z
                                  float,  # radiometry
                                  float,  # inclin
                                  float,  # gammaup
                                  float,  # gammadown
                                  float,  # cpsup
                                  float  # cpsdown
                                  )
    closed = QtCore.pyqtSignal()
    opened = QtCore.pyqtSignal()

    def __init__(self, parent=None):
        """Constructor."""
        super(RTKubeDockWidget, self).__init__(parent)
        # Set up the user interface from Designer.
        # After setupUI you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)

        self.ser = rtkubeSerial()

        self.ser.sendGNSS.connect(self.setDataGNSS)
        self.ser.sendGammaInc.connect(self.setDataGammaInc)
        self.ser.sendError.connect(self.errors)

        self.connectButton.clicked.connect(self.connection)

        self.isConnected = False

        self.lastAlertIncli = time.time()

        self.data = list()

        self.logger = logging.getLogger('rtkubeSerial_gui')
        self.logger.setLevel(logging.DEBUG)
        fh = logging.handlers.TimedRotatingFileHandler(LOGFILE + '2')
        fh.setLevel(logging.DEBUG)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)
        self.logger.addHandler(fh)

        self.settings = QtCore.QSettings()

    def closeEvent(self, event):
        self.closingPlugin.emit()
        event.accept()

    def clear(self):
        self.GNSSDate.clear()
        self.GNSSTime.clear()
        self.GNSSLat.clear()
        self.GNSSLon.clear()
        self.GNSSAlt.clear()
        self.GNSSPlaniPrec.clear()
        self.GNSSAltiPrec.clear()
        self.GNSSCalculation.clear()
        self.X.clear()
        self.Y.clear()
        self.Z.clear()
        self.ZFixed.clear()
        self.gammaGUp.clear()
        self.gammaTUp.clear()
        self.gammaGLow.clear()
        self.gammaTLow.clear()
        self.gammaInc.clear()
        self.qualRadiometry.clear()
        self.qualRadiometry.setStyleSheet('background-color:' + 'rgb(119, 119, 119)')

    def setDataGNSS(self, date, time, lon, lat, alt, prPlan, prAlti, calc):
        if not self.isConnected:
            return

        def convertToEpsg(x, y):
            geom = QgsGeometry.fromPointXY(QgsPointXY(x, y))
            trans = QgsCoordinateTransform(
                QgsCoordinateReferenceSystem(4326), QgsCoordinateReferenceSystem(EPSG), QgsProject.instance())
            geom.transform(trans)
            pt = geom.asPoint()
            return (pt.x(), pt.y())

        self.GNSSDate.setText(date)
        self.GNSSTime.setText(time)
        self.GNSSLat.setText(lat)
        self.GNSSLon.setText(lon)
        self.GNSSAlt.setText(alt)
        self.GNSSPlaniPrec.setText(prPlan)
        self.GNSSAltiPrec.setText(prAlti)

        calculation = GPS_CALCULATION[int(calc)]
        self.GNSSCalculation.setText(calculation)

        gpsmode = self.settings.value('/GeoCube/GPSmode', 'Fixed')
        if gpsmode not in GPS_CALCULATION.values():
            gpsmode = 'Fixed'
        self.GNSSCalculation.setStyleSheet('color: ' + GPS_COLOR[int(calc)])
        if int(calc) >= list(GPS_CALCULATION.keys())[list(GPS_CALCULATION.values()).index(gpsmode)]:
            # no data or radio, incli + gammaup, gammadown, cpsup, cpsdown
            if (len(self.data) == 0 or len(self.data) == 6):
                lonTrans, latTrans = convertToEpsg(float(lon), float(lat))
                altTrans = float(alt) - 2
                self.X.setText(str(round(lonTrans, 3)))
                self.Y.setText(str(round(latTrans,3)))
                self.Z.setText(alt)
                self.ZFixed.setText(str(round(altTrans,2)))
                self.data = [date + " " + time, lonTrans,
                             latTrans, altTrans] + self.data
            else:
                self.logger.warning('Incoherent data Actual: {} Received: {}'.format(
                    self.data, [date, time, lon, lat, alt, prPlan, prAlti, calc]))
                self.data[:] == []
            if len(self.data) == NB_FIELDS_DATA:
                self.logger.debug('Send: {}'.format(self.data))
                self.dataToSMU.emit(*self.data)
                self.data[:] = []
        else:
            self.data[:] = []
            self.logger.warning('Incoherent data. Data is not fixed')

    def setDataGammaInc(self, upGamma, lowGamma, incli):
        if not self.isConnected:
            return

        inclination = float(incli) / 10.0
        self.gammaGUp.setText(upGamma)
        self.gammaGLow.setText(lowGamma)
        self.gammaInc.setText(str(inclination))

        teneurUp = int(upGamma) / float(self.settings.value('/GeoCube/gammaup',
                                                            1000.00))
        teneurLow = int(lowGamma) / float(self.settings.value('/GeoCube/gammalow',
                                                              1000.00))
        self.setQualRadiometry(teneurUp)
        self.gammaTUp.setText(str(round(teneurUp,3)))
        self.gammaTLow.setText(str(round(teneurLow,3)))
        if inclination <= float(self.settings.value('/GeoCube/angle', 10)):
            if len(self.data) == (NB_FIELDS_DATA - 6):  # 6 - upGamma - Inclination
                cps = [int(upGamma), int(lowGamma), teneurUp, teneurLow]
                self.data = self.data + \
                    [teneurUp, inclination] + cps
            else:
                self.logger.warning('Incoherent data Actual: {} Received: {}'.format(
                    self.data, [upGamma, inclination]))
                self.data[:] == []
            if len(self.data) == NB_FIELDS_DATA:
                self.logger.debug('Send: {}'.format(self.data))
                self.dataToSMU.emit(*self.data)
                self.data[:] = []
        else:
            self.logger.warning('Max inclination reached')
            if self.lastAlertIncli < time.time() - DURATION:
                iface.messageBar().pushMessage("RTKube", self.tr("Max inclination reached"),
                                               level=Qgis.Warning, duration=DURATION)
                self.lastAlertIncli = time.time()
            self.data[:] = []

    def setQualRadiometry(self, rad):
        self.qualRadiometry.setStyleSheet('background-color:' + 'rgb(119, 119, 119)')
        if rad <= 0.6:
            self.qualRadiometry.setText('M1')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[1])
        elif rad > 0.6 and rad <= 0.9:
            self.qualRadiometry.setText('M2')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[2])
        elif rad > 0.9 and rad <= 1.2:
            self.qualRadiometry.setText('M3')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[3])
        elif rad > 1.2 and rad <= 1.8:
            self.qualRadiometry.setText('M4')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[4])
        elif rad > 1.8 and rad <= 3.0:
            self.qualRadiometry.setText('M5')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[5])
        elif rad > 3.0:
            self.qualRadiometry.setText('M6')
            self.qualRadiometry.setStyleSheet('font: 28pt "Arial Black"; background-color:' +
                                              QUAL_COLOR[6])

    def connection(self):
        if self.isConnected == False:
            self.isConnected = True
            self.ser.connect()
            self.opened.emit()
        else:
            answer = QtWidgets.QMessageBox.question(self, self.tr('Confirm disconnection'),
                                                    self.tr('Do you want to disconnect the CAN-OP'))
            if answer == QtWidgets.QMessageBox.No:
                self.isConnected = True
                return
            self.disconnection()

        self.changeButtonText()

    def disconnection(self):
        self.isConnected = False
        self.ser.disconnect()
        self.closed.emit()
        self.clear()

    def changeButtonText(self):
        if self.isConnected:
            self.connectButton.setText(self.tr("Disconnection"))
        else:
            self.connectButton.setText(self.tr("Connection"))

    def errors(self, msg):
        iface.messageBar().pushMessage(
            "RTKube", msg, level=Qgis.Critical, duration=DURATION)
        self.disconnection()
        self.changeButtonText()
