# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os

from PyQt5 import QtWidgets, uic

import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar

from qgis.core import QgsGeometry

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'chart.ui'))


class CtrlCraftOfficeEstimatedBlocsChart(QtWidgets.QDialog, FORM_CLASS):

    """
    Displays a correlation point for each slab within the current
    deposit/panel/blast with its corresponding radiometry and the
    corresponding estimated bloc's radiometry.
    """

    def __init__(self, data, canvas, parent=None):
        super(CtrlCraftOfficeEstimatedBlocsChart, self).__init__(parent)
        self.setupUi(self)
        self.data = data
        self.canvas = canvas

    def init(self, deposit, panel, blast):
        self.__init_chart(deposit, panel, blast)

    def zoomOnEstimatedBlocs(self, event):
        index = event.ind
        sample_number = self.currentIdEstimatedBlocss[index[0]]
        wkt = self.data.estimated_bloc_wkt(sample_number)
        geom = QgsGeometry.fromWkt(wkt)

        self.canvas.setExtent(geom.boundingBox())
        self.canvas.zoomOut()

    def __init_chart(self, deposit, panel, blast):
        records = self.data.estimated_blocs_stats_for_slabs(deposit, panel, blast)

        # Match the data
        dataChartSlabGamma = []
        dataChartEstimatedBlocs = []
        self.currentIdEstimatedBlocss = []
        for r in records:
            valSlabGamma = float(r[0]) if r[0] else 0.0
            valEstimatedBlocs = float(r[1]) if r[1] else 0.0

            dataChartEstimatedBlocs.append(valSlabGamma)
            dataChartSlabGamma.append(valEstimatedBlocs)

            self.currentIdEstimatedBlocss.append(r[2])

        # Build chart
        fig, ax = plt.subplots()
        ax.scatter(dataChartEstimatedBlocs, dataChartSlabGamma, c='r', s=30, alpha=0.5,
                   picker=5)

        ax.set_xlabel(self.tr('Estimated blocs grade'))
        ax.set_ylabel(self.tr('Gamma slab grade'))
        ax.set_title(self.tr('Correlation gamma slab/ estimated blocs'))

        ax.grid(True)
        fig.tight_layout()

        chartCanvas = FigureCanvas(fig)
        chartCanvas.mpl_connect('pick_event', self.zoomOnEstimatedBlocs)

        # Navigation widget
        toolbar = NavigationToolbar(chartCanvas, self)

        # set the layout
        self.vLayout.addWidget(toolbar)
        self.vLayout.addWidget(chartCanvas)
