# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from builtins import next
import os
import csv

from PyQt5 import uic
from PyQt5.QtWidgets import (QFileDialog,
                         qApp,
                         QLabel,
                         QDialog)

from qgis.gui import QgsMessageBar
from qgis.core import Qgis

from CtrlCraft import CtrlCraftLogger

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'gate_import.ui'))


class CtrlCraftOfficeGateImport(QDialog, FORM_CLASS):
    """
    Dialog to import Slabs in database.
    """

    def __init__(self, data, parent=None):
        super(CtrlCraftOfficeGateImport, self).__init__(parent)
        self.setupUi(self)
        self.data = data
        self.updated = False

        self.mCancelButton.clicked.connect(self.__close)
        self.mImportButton.clicked.connect(self.__import)
        self.mSelectButton.clicked.connect(self.__select)

        self.mProgressBar.setValue(0)
        self.mLabel.setText('')

    def __close(self):
        self.close()

    def __select(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)
        dlg.setNameFilter("CSV files (*.txt *.csv)")

        filenames = []
        if dlg.exec_():
            filenames = dlg.selectedFiles()

        if filenames:
            filename = filenames[0]

            # read header to fill combobox
            with open(filename, 'r') as csvfile:
                reader = csv.reader(csvfile, delimiter=',')
                header = next(reader)

                if self.__is_valid(header):
                    self.mLineEdit.setText(filename)
                else:
                    CtrlCraftLogger().logBar('Invalid format', Qgis.Critical)

    def __import(self):
        filename = self.mLineEdit.text()
        if not filename:
            return

        self.mSelectButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        self.mImportButton.setEnabled(False)

        self.mLabel.setText('Insert gate data in database...')
        self.mProgressBar.setValue(0)
        qApp.processEvents()

        nlines = 0
        with open(filename, 'r') as csvfile:
            nlines = len(list(csv.reader(csvfile, delimiter=','))) - 1

        new_data = 0
        with open(filename, 'r') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')

            nline = 0
            header = None
            for row in reader:
                if not header:
                    header = True
                    continue

                deposit = row[0]
                panel = row[1]
                blast = row[2]
                truck = row[3]
                date = row[4]
                time = row[5]
                radiometry = row[6]

                datetime = '{} {}:00'.format(date.replace('/', '-'), time.replace('h', ':'))

                if not self.data.gate_data(truck, datetime):
                    self.data.gate_create(deposit, panel, blast, truck, datetime, radiometry)
                    new_data += 1

                # Update progress bar
                nline += 1
                total = nline*100/nlines
                self.mProgressBar.setValue(total)

            self.mProgressBar.setValue(100)

            msg = ('{} gate data imported with success ({} ignored)!'
                   .format(new_data, nlines-new_data))
            self.mLabel.setText(msg)

            self.mProgressBar.setMaximum(100)
            self.mProgressBar.setValue(100)

            self.updated = True

        self.mCancelButton.setEnabled(True)

    def __is_valid(self, header):
        valid_header = [
            'deposit',
            'panel',
            'blast',
            'truck',
            'date',
            'time',
            'radiometry']

        return header == valid_header
